#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: nu:ai:ts=4:sw=4

#
#  Copyright (C) 2025 Joseph Areeda <joseph.areeda@ligo.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
""""""

__author__ = 'joseph areeda'
__email__ = 'joseph.areeda@ligo.org'

import argparse
import socket
from pathlib import Path, PosixPath

from omicron_utils.omicron_config import OmicronConfig
import pytest

# these command line arguments came from debugger breakpoint in pyomicron running on Joe's
# home workstation



class TestOmicronJob:

    @classmethod
    def setup_class(self):
        fqdn = socket.getfqdn()
        print(f'\nRunning tests on {fqdn} cwd: {Path.cwd()}')

        self.dummy_args_home = {'archive': False, 'auth_type': 'x509', 'cache_file': None,
                            'conda_env': 'ligo-omicron-3.10',
                            'condor_accounting_group': 'ligo.prod.o4.detchar.transient.omicron',
                            'condor_accounting_group_user': 'areeda', 'condor_command': [],
                            'condor_request_disk': '20G', 'condor_retry': 2,
                            'config_file': '/Volumes/Data2/omicron/omicron/online/h1-channels.ini',
                            'dagman_option': ['force', 'import_env'], 'exclude_channel': [],
                            'executable': '/Users/areeda/miniforge3/envs/ligo-omicron-3.10-test/bin/omicron',
                            'file_tag': 'PEM1', 'gps': None, 'group': 'PEM1', 'ifo': 'H1',
                           'log_file': None, 'max_channels_per_job': 10, 'max_chunks_per_job': 4,
                           'max_concurrent': 64, 'max_online_lookback': 43200, 'no_segdb': False,
                           'no_submit': True, 'output_dir': PosixPath('/Users/areeda/t/omicron/online-h1'),
                           'reattach': False, 'rescue': False, 'skip_gzip': False, 'skip_hdf5_merge': False,
                           'skip_ligolw_add': False, 'skip_omicron': False, 'skip_postprocessing': False,
                           'skip_rm': False,
                           'skip_root_merge': False, 'submit_rescue_dag': 0, 'universe': 'vanilla',
                           'use_dev_shm': False,
                           'verbose': 5}

        config_path = OmicronConfig.get_default_config_path()
        assert config_path is not None

    @classmethod
    def test_getsubmit(self):
        '''
        Simulate using config file, command line arguments, etc. to generate the parameters for our submit file
        :return: assert results are produced tested at least a little
        '''
        args = argparse.Namespace(**dummy_args_home)
        config_path = OmicronConfig.get_default_config_path()
        assert config_path is not None
        home = str(Path.home().absolute())
        symbols = {'HOME': home}
        pro = ''




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: nu:ai:ts=4:sw=4

#
#  Copyright (C) 2025 Joseph Areeda <joseph.areeda@ligo.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Class to facilitate using ANSI terminal formatting codes
"""

__author__ = 'joseph areeda'
__email__ = 'joseph.areeda@ligo.org'

ansi_term_codes = {
    'reset': "\x1b[m",  # ANSI CODE 0   → resets all styles, it's the same of \x1b[0m

    'bold': "\x1b[1m",  # ANSI CODE 1   → increases intensity, with a slight color change
    'rbold': "\x1b[22m",  # ANSI CODE 22  → resets bold or dim (they are mutually exclusive styles)

    'dim': "\x1b[2m",  # ANSI CODE 2   → decreases intensity, with a slight color change
    'rdim': "\x1b[22m",  # ANSI CODE 22  → resets bold or dim (they are mutually exclusive styles)

    'italic': "\x1b[3m",  # ANSI CODE 3   → italic
    'ritalic': "\x1b[23m",  # ANSI CODE 23  → resets italic

    'underline': "\x1b[4m",  # ANSI CODE 4   → underline
    'runderline': "\x1b[24m",  # ANSI CODE 24  → resets underline or doubleunderline (they are mutually exclusive styles)

    'doubleunderline': "\x1b[21m",  # ANSI CODE 21  → double underline (not supported by Konsole)
    'rdoubleunderline': "\x1b[24m",
    # ANSI CODE 24  → resets underline or doubleunderline (they are mutually exclusive styles)

    'curlyunderline': "\x1b[4:3m",  # ANSI CODE 4:3 → curly underline (not supported by Konsole)
    'rcurlyunderline': "\x1b[4:0m",  # ANSI CODE 4:0 → resets curly underline

    'blink': "\x1b[5m",  # ANSI CODE 5   → blink
    'rblink': "\x1b[25m",  # ANSI CODE 25  → resets blink

    'reverse': "\x1b[7m",  # ANSI CODE 7   → swaps text and background colors
    'rreverse': "\x1b[27m",  # ANSI CODE 27  → resets reverse

    'hidden': "\x1b[8m",  # ANSI CODE 8   → characters not displayed, helpful for passwords
    'rhidden': "\x1b[28m",  # ANSI CODE 28  → resets hidden

    'strikethrough': "\x1b[9m",  # ANSI CODE 9   → characters crossed by a central line
    'rstrikethrough': "\x1b[29m",  # ANSI CODE 29  → resets strikethrough

    'overline': "\x1b[53m",  # ANSI CODE 53 → overline
    'roverline': "\x1b[55m",  # ANSI CODE 55 → resets overline
}
ansi_color_codes = {
    # colors
    'yellowText': "\x1b[38;2;255;255;0m",
    'yellowBg': "\x1b[48;2;255;255;0m",
    'redText': "\x1b[38;2;255;0;0m",
    'redBg': "\x1b[48;2;255;0;0m",
    'greenText': "\x1b[38;2;0;255;0m",
    'greenBg': "\x1b[48;2;0;255;0m",
    'blueText': "\x1b[38;2;100;100;255m",
    'blueBg': "\x1b[48;2;0;0;255m",
    'blackText': "\x1b[38;2;0;0;0m",
    'blackBg': "\x1b[48;2;0;0;0m",
    'whiteText': "\x1b[38;2;255;255;255m",
    'whiteBg': "\x1b[48;2;255;255;255m",
}


def format_str(fmt_code, string):
    """
    Add Ansi codes to format a string and reset afterward
    :param str | list[str] fmt_code: one or more available format codes
    :param str string: what to format
    :return str: what to print after formatting eg: <start format>string<reset format>
    """
    ret = string
    if fmt_code is not None:
        if isinstance(fmt_code, str):
            fmt_list = [fmt_code]
        else:
            fmt_list = fmt_code

        rfmt = ""
        sfmt = ''
        err_fmt = ''

        for fmt in fmt_list:
            if fmt in ansi_term_codes.keys():
                sfmt += ansi_term_codes[fmt]
                rfmt += ansi_term_codes['r' + fmt]
            else:
                err_fmt += fmt + ', '

        if err_fmt != '':
            raise KeyError(f' Unknown format code(s) {err_fmt}')

        ret = f'{sfmt}{string}{rfmt}'

    return ret


def get_color(color, fg_bg='fg'):
    """
    Translate color to ansi color escape code
    :param str color: known color or 0xRRBBGG
    :param str fg_bg: "fg" or "bg" for foreground or background color
    :return str: ansi color escape code
    """
    if color is None:
        ret = ''
    elif isinstance(color, int) or color.startswith("0x"):
        c = int(color)
        r = (c & 0xFF0000) >> 16
        g = (c & 0x00FF00) >> 8
        b = (c & 0x0000FF)

        code = 38 if fg_bg.lower() == "fg" else 48
        ret = f"\x1b[{code};2;{r};{g};{b}m"
    else:
        if fg_bg.lower() == "bg":
            color_code = f'{color}Bg'
        elif fg_bg.lower() == "fg":
            color_code = f'{color}Text'
        else:
            raise KeyError(f' Unknown foreground/background code(s) {fg_bg}')
        if color_code not in ansi_color_codes.keys():
            raise KeyError(f' Unknown color code(s) {color}. You may specify colors as"0xRRGGBB"')
        ret = ansi_color_codes[color_code]
    return ret


def ansi_color_text(string, color=None, backgrond=None, reset=True):
    """
    Add color codes to a string, reset optionally inserted
    Colors can be Red, Green, Blue, Black, White, Yellow, or 0xRRGGBB
    :param str string: what to color
    :param str | int | None color: foreground color
    :param str | int |None backgrond: background color
    :param bool reset: reset option
    :return str: string with color codes for printng
    """

    fg = get_color(color, 'fg')
    bg = get_color(backgrond, 'bg')
    ret = f'{fg}{bg}{string}' + ansi_term_codes['reset'] if reset else ''
    return ret



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: nu:ai:ts=4:sw=4

#
#  Copyright (C) 2024 Joseph Areeda <joseph.areeda@ligo.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
""""""

__author__ = 'joseph areeda'
__email__ = 'joseph.areeda@ligo.org'

from omicron_utils.conda_fns import get_conda_run
from omicron_utils.omicron_config import OmicronConfig


def test_config():
    """
    Omicron control program configration, not the Omcrong trigger generator's config
    :return: None
    """
    omicron_config = OmicronConfig()
    assert omicron_config is not None

    config = omicron_config.get_config(save_if_none=False)
    getenv = config['condor']['getenv']
    assert getenv is not None
    pass


def test_get_conda_run():
    """
    try to get conda prefix
    :return: None
    """
    omicron_config = OmicronConfig()
    assert omicron_config is not None

    config = omicron_config.get_config(save_if_none=False)

    env1 = 'ligo-omicron-3.10'
    prefix = get_conda_run(config, env1)
    assert prefix is not None
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: nu:ai:ts=4:sw=4

#
#  Copyright (C) 2025 Joseph Areeda <joseph.areeda@ligo.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
A class to combine parameter from different sources to create a
"""

__author__ = 'joseph areeda'
__email__ = 'joseph.areeda@ligo.org'


class OmicronCondorJob:
    """"
    This class will use multiple sources to generate statements for condor submit files.
    The order of sources is important.
    1. The [condor] section of the config file
    2. The [<program>] section of the config file
    3. Specific command line arguments

    Variable substitution syntax %{<variable name>}. It uses the environment variables ad a user supplied dictionary
    the user dict will take precedence if duplicates exist.
    """

    def __init__(self, config, program, args=None, symbols=None):
        """

        :param  configparser.ConfigParser config: The ligo_omicron config (not the omicron channel config)
        :param str program: program name as used for config section
        :param namespace args: argparser args
        :param dict[str, str] symbols: symbol table used in substitutions
        """
        self.config = config
        self.program = program
        self.args = vars(args) if args else dict()
        self.symbols = symbols if symbols is not None else dict()

    def get_classads(self):
        """
        Combine the layers of inputs to produce
        :return dict[str, str]: The Classad
        """
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: nu:ai:ts=4:sw=4

#
#  Copyright (C) 2024 Joseph Areeda <joseph.areeda@ligo.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Set of functions supporting use of conda
"""

__author__ = 'joseph areeda'
__email__ = 'joseph.areeda@ligo.org'

import logging
import os
import re
import shutil
import subprocess
from os import getenv

PREFIX_SEARCH_LIST = [
    '${home}/.conda/envs/ligo-summary-3.10',
    '${home}/.conda/envs/ligo-summary-3.10-test',
    '/cvmfs/software.igwn.org/conda/envs/igwn',
    '${home}/mambaforge/envs/ligo-omicron-3.10',
    '${home}/mambaforge/envs/ligo-omicron-3.10-test',
    ''
]


def get_conda_run(config, env=None, logger=None):
    """
    Determine what is needed to run a command in the specified conda environment
    :param configparser.ConfigParser config: our pipeline configuration
    :param str env: cona env name or path
    :param logging.Logger logger:
    :return str, str: path to executable, run arguments
    """

    if logger is None:
        logger = logging.getLogger('get_conda_run')
        logger.setLevel(logging.INFO)

    executable = get_conda_exe()

    ermsg = 'conda program not found' if executable is None else ''

    if env is None:
        env = getenv('CONDA_PREFIX')

    if env is None and config.has_option('conda', 'environment'):
        env = config['conda']['environment']

    if env is None:
        ermsg += 'Unable to determine conda environment for Omicron pipeline'

    if ermsg:
        raise ValueError(ermsg)

    arguments = 'run '
    if '/' in env:
        # assume we have a path
        arguments += f'--prefix {env} '
    else:
        prefix = conda_name_to_prefix(env)
        if prefix is None:
            raise ValueError(f'Unable to find conda prefix for {env}')
        arguments += f'--prefix {prefix} '

    arguments += '--no-capture-output '

    return executable, arguments


def get_conda_exe():
    conda = os.getenv('CONDA_EXE')
    if conda is None:
        conda = shutil.which('conda')
    if conda is None:
        conda = shutil.which('mamba')

    if conda is None:
        raise ValueError('Unable to find conda executable')
    return conda


def conda_name_to_prefix(conda_name: str, or_current=True) -> str:
    """
    Find the prefix of a conda environment from its name or current environment
    :param str conda_name: conda environment name
    :param bool or_current: if True, return current conda environment if set
    :return str: prefix of conda environment

    :param conda_name:
    :return:
    """
    conda =get_conda_exe()

    cmd = [conda, 'info', '--envs']
    res = subprocess.run(cmd, capture_output=True)

    ret = None

    env_match = re.compile('(\\S+)([\\s\\*]+)(.*)$')
    for line in res.stdout.decode('utf-8').splitlines():
        m = env_match.match(line)
        if m:
            name = m.group(1)
            star = m.group(2)
            prefix = m.group(3)
            if star.find('*') >= 0 and or_current:
                ret = prefix
                break
            elif name == conda_name:
                ret = prefix
                break
            elif prefix.find(conda_name) >= 0:
                ret = prefix

    return ret



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: nu:ai:ts=4:sw=4

#
#  Copyright (C) 2024 Joseph Areeda <joseph.areeda@ligo.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
""""""

__author__ = 'joseph areeda'
__email__ = 'joseph.areeda@ligo.org'

import configparser
import logging
import os
import re
import sys
from pathlib import Path


class OmicronConfig:
    """
    Representing the configuration of all LSC Omicron programs in one place.
    The hierarchy of where to get the information. First one of ther list found.
    1. File path passed in creation or in set_file method
    2. Environment variable OMICRON_CONFIG
    3.
    4. Internal defaults. See ligoomicron.ini in this project.

    Our philosophy is to put as many options as we can into the configuration.
    """

    @staticmethod
    def get_default_config_path():
        """
        Standardize where we look for our config file if not specified
        :return str: absolute path to config file
        """
        default_path = Path.home() / '.ligo_omicron.ini'
        return str(default_path.absolute())

    @staticmethod
    def get_internal_config_path():
        """
        The internal config file is our best guess for default values at release time
        :return str: absolute path to the config file
        """
        me = Path(__file__)
        mydir = me.parent
        path = mydir / 'ligoomicron.ini'
        return str(path.absolute())

    def __init__(self, path=None, logger=None):
        """

        :param Path|str|None path: path to configuration file
        :param logging.Logger logger: program logger
        """
        self.config = None
        if logger is None:
            # global logger
            log_file_format = "%(asctime)s - %(levelname)s - %(funcName)s %(lineno)d: %(message)s"
            log_file_date_format = '%m-%d %H:%M:%S'
            logging.basicConfig(format=log_file_format, datefmt=log_file_date_format)
            self.logger = logging.getLogger('OmicronConfig')
            self.logger.setLevel(logging.INFO)
        else:
            self.logger = logger
        if path is None:
            self.path = self.get_default_config_path()
        else:
            self.path = Path(path)
            self.set_file(path)

    def set_file(self, path):
        """
        Read the configuration file replacing any existing values
        :param Path|str path: path to configuration file
        :return configparser.ConfigParser: copy of the config
        """
        if path is None or not Path(path).is_file() or not Path(path).exists():
            raise FileNotFoundError(f'{path} does not exist')
        self.path = Path(path)
        self.config = configparser.ConfigParser()
        self.config.read(self.path)
        self.logger.debug(f'Configuration read from {self.path.absolute()}')
        return self.config

    def get_config(self, save_if_none=True):
        """
        The preferred way to get current configuration. This method looks through list of places we can use
        for configuration file
        :param bool save_if_none: if we don't have confi in home add it
        :return configparser.ConfigParser:

        """
        default_path = Path(OmicronConfig.get_default_config_path())
        if self.config is None:
            path = os.getenv('OMICRON_CONFIG', None)
            if path is None:
                path = default_path
            if path is None or not path.exists():
                path = OmicronConfig.get_internal_config_path()
            path = Path(path)
            self.logger.debug(f'Read config file {path.absolute()}')

            if path.exists():
                config = configparser.ConfigParser()
                config.read(path)
                self.config = config
                self.path = path
                # remove line breaks from  continuation lines
                for section in self.config.sections():
                    for option in self.config.options(section):
                        new_option = config[section][option]
                        new_option_lst = [item for item in new_option.strip().split('\n')]
                        new_option = str(" ".join(new_option_lst))
                        new_option = new_option.replace('\\', '')
                        self.config[section][option] = str(new_option)

                if not default_path.exists() and save_if_none:
                    with default_path.open('w') as cfp:
                        self.config.write(cfp)
            else:
                raise FileNotFoundError('No configuration file found')
        return self.config

    def print_config(self, out=None):
        """
        pretty print the current configuration
        :param str | Path | None out: output file path, None for stdout
        :return: None
        """
        try:
            if out is None:
                print(f'Omicron configuraton in file: {self.path.absolute()}\n')
                self.config.write(sys.stdout)
            else:
                with Path(out) as fp:
                    self.config.write(fp)
        except (TypeError, FileNotFoundError, IOError, IsADirectoryError, OSError) as e:
            self.logger.error(f'Error while printing configuration: {e}')

    def get_option(self, section, option):
        """
        Return an option in the configuration we have read handling multi-line
        :param str section: section name
        :param str option: option name
        :return str: option value
        """
        ret = self.config.get(section, option)
        ret = ret.strip()
        ret = re.sub('\\s+', ' ', ret)
        return ret



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: nu:ai:ts=4:sw=4

#
#  Copyright (C) 2025 Joseph Areeda <joseph.areeda@ligo.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Provides functions to query job attributes in an HTCondor batch processing system.

This module contains utility functions to filter and retrieve job information from
an HTCondor batch processing scheduler. Queries can be performed using regex patterns,
cluster IDs, or constraints for more targeted results. Results are formatted according
to the specified projections, enabling flexible data extraction.
"""

__author__ = 'joseph areeda'
__email__ = 'joseph.areeda@ligo.org'

import getpass
import platform
from pathlib import Path
from time import sleep, time

import htcondor2 as htcondor
from htcondor2 import HTCondorException
from pythia8 import theta


def get_schedd(schedd_spec=None):
    """
    Retrieve an htcondor.Schedd instance based on the provided specification.

    This function returns an instance of `htcondor.Schedd`. When no specification
    is provided, it defaults to return the default `htcondor.Schedd` object.

    :param str|None schedd_spec: The specification for the scheduler usually a fqdn. If `None`, the default
        `htcondor.Schedd` instance is returned. Optional.
    :return: An `htcondor.Schedd` instance based on the provided specification.
    """
    schedd_spec = None
    for i in range(10):
        try:
            if schedd_spec is None:
                schedd_spec = htcondor.Schedd()
            else:
                schedd_spec = htcondor.Schedd(schedd_spec)
            break
        except Exception as e:
            print(f'Try #{i+1}: error connecting to schedd: {e}')
            sleep(10)
    if schedd_spec is None:
        raise HTCondorException('Unable to connect to schedd')
    return schedd_spec


def schedd_query(constraint=None, schedd=None, projection=None,):
    """
        Query scheduler for jobs
    :param string constraint: Constrint expression eg: "ClusterID == 1234"
    :param htcondor2._schedd.Schedd|str|None schedd: Schedd object or name of schedd or None for default
    :param str|None projection: Projection expression eg: "ClusterId, ProcId, JobBatchName, JobStatus
    :return list[dict]: matching jobs, may be empty
    """
    schedd = get_schedd(schedd)

    if projection is None:
        projection = ["ClusterId", "ProcId", "JobBatchName", "JobStatus"]

    ret = schedd.query(projection=projection, constraint=constraint)
    ret = list(ret) if ret else []
    return ret


def query_by_name(name, schedd=None, projection=None,):
    """
    Query the schedd for the value of a classad attribute
    :param str name: regex pattern to match against JobBatchName. EG: '.*online.*PEM1.*'
    :param htcondor.Schedd schedd: schedd to query
    :param list[str] projection: A list of classad attributes to return,
                                 defaults to ["ClusterId", "ProcId", "JobBatchName", "JobStatus"]
    :return: list of matching Job attributes containing the name
    """

    constraint = f'regexp("{name}", JobBatchName)'
    return schedd_query(constraint=constraint, schedd=schedd, projection=projection)


def query_by_clusterid(clusterid, procid=None, schedd=None, projection=None, ):
    """
    Retrieves information about a specific cluster of jobs in a batch processing scheduler.
    This function queries a scheduler using the provided cluster ID and retrieves relevant
    job information defined by the projection.

    :param clusterid: The unique cluster ID used to identify the batch of jobs.
    :type clusterid: int

    :param procid: The unique process ID used to identify a specific job within a cluster.
    :type procid: int

    :param schedd: The scheduler object to perform the query on. If not provided, a
        default scheduler will be used. Optional.
    :type schedd: object, optional

    :param projection: The list of attributes to include in the query result. Defaults
        to ["ClusterId", "ProcId", "JobBatchName", "JobStatus"].
    :type projection: list[str]

    :return: A list of job details matching the given cluster ID.
    :rtype: list[dict]
    """

    cid = clusterid
    constraint = f'ClusterId == {cid}'
    if procid and '.' not in str(clusterid):
        constraint += f' && ProcId == {procid}'
    if projection is None:
        projection = ["ClusterId", "ProcId", "JobBatchName", "JobStatus"]
    qres = schedd_query(constraint=constraint, schedd=schedd, projection=projection)
    return qres

def wait_for_job(job_str, schedd=None, sleep_time=5, timeout=None, ):
    """
    Waits for the completion of a given job.

    This function monitors the status of a given job and waits until the job
    is completed. Use this function when there is a need to ensure that the
    execution of the code can only proceed after the job has finished.

    :param job_str: The job name or cluster id whose completion status is to be monitored.
    :type job_str: str|number
    :param sleep_time: polling interval in seconds
    :type sleep_time: int
    :param schedd: The scheduler object to perform the query on. If not provided, a
    :type schedd: object or schedd name, optional
    :param timeout: timeout in seconds
    :type timeout: int
    :return: Returns when thejob is no longer found in the scheduler
    :rtype: None
    """
    start = time()
    job_str = str(job_str)
    while query_by_clusterid(job_str, schedd=schedd) or query_by_name(job_str, schedd=schedd):
        sleep(sleep_time)
        elapsed = time() - start
        if timeout and elapsed >= timeout:
            raise TimeoutError(f'Timeout waiting for job {job_str} to complete')


def condor_submit_job(submit_file, schedd=None):
    """
    Submit a job to an HTCondor scheduler using a specified submit file.

    This function reads the content of the submit file to create a Submit object,
    then queues the job in the HTCondor scheduler provided. It returns the cluster ID
    of the submitted job and the results of the job submission process.

    :param submit_file: The path to the HTCondor submit file. It contains the
        job's description and requirements.
    :type submit_file: str|Path
    :param schedd: The HTCondor Schedd object to which the job will be submitted.
        If not specified, a default scheduler is used.
    :type schedd: optional[htcondor.Schedd]
    :return: A tuple containing the cluster ID of the submitted job as an integer
        and the submission result as an object indicating the details of the submission.
    :rtype: tuple[int, object]
    """

    submit_path = Path(submit_file)
    # Load the submit spec file
    with submit_path.open() as f:
        submit_description = f.read()

    # Parse it into a Submit object
    submit = htcondor.Submit(submit_description)

    cluster_id, submit_result = condor_queue_job(submit, schedd=schedd)
    return cluster_id, submit_result


def condor_queue_job(submit, count=1, schedd=None):
    """
    Submit a job or multiple jobs to the Condor scheduler.

    This function allows submission of one or more jobs to a Condor scheduler,
    returning the cluster ID associated with the job(s) and the submission result.
    A scheduler is optionally specified; otherwise, a default scheduler is used.

    :param submit: The job submission object defining job parameters and configurations.
    :param count: An optional integer specifying the number of jobs to submit,
        with a default of 1.
    :param schedd: An optional Condor scheduler object. If not provided,
        a default scheduler is retrieved and utilized.
    :return: A tuple where the first element is the cluster ID associated with
        the submission and the second element is the submission result.
    """
    schedd = get_schedd(schedd)
    if platform.machine() == 'arm64':
        submit.update({'requirements': '(Target.ARCH == "X86_64" || Target.ARCH == "ARM64")'})

    if count > 1:
        res = schedd.submit(submit,count=count)
    else:
        res = schedd.submit(submit)
    return res.cluster(), res

def condor_submit_dag(dag_file, schedd=None, options=True, use_strict=3, name=None,):
    """
    Submit a DAG job to HTCondor with optional configurations.

    This function allows for the submission of Directed Acyclic Graph (DAG)
    jobs to an HTCondor system. The DAG submission file can be customized
    with additional options, strictness modes can be set for `dagman`, and a
    specific batch name can optionally be provided.

    :param dag_file: Path to the DAG file to be submitted.
    :type dag_file: str|Path
    :param schedd: HTCondor schedd object to which the job will be submitted.
                   If not provided, the default scheduler is used.
                   Optional.
    :type schedd: htcondor.Schedd or None
    :param options: Either a dictionary of additional submission options
                    or a boolean indicating whether to use default options.
                    Optional.
    :type options: dict or bool
    :param use_strict: Strictness level for DAGMan, where the default value
                       is set to '3'.
                       Optional.
    :type use_strict: int
    :param name: (Optional) Custom batch name for submission operations
                 overriding the default DAG basename.
    :type name: str or None
    :return: Tuple containing the cluster ID of the submitted DAG job
             and the submission result information.
    :rtype: tuple[int, dict]
    """
    dag_path = Path(dag_file)
    if not dag_path.exists():
        raise FileNotFoundError(f'DAG file: {dag_file} does not exist')

    if name is not None:
        batch_name = name
    else:
        batch_name = f'{dag_path.stem} $(ClusterID)'
    default_options = {'batch-name': batch_name, 'force':  'True', 'import_env': 'True'}

    try:
        if type(options) == dict:
            dag_sub = htcondor.Submit.from_dag(str(dag_path.absolute()), options=options)
        elif type(options) == bool and options:
            if options:
                dag_sub = htcondor.Submit.from_dag(dag_file, options=default_options)
        else:
            dag_sub = htcondor.Submit.from_dag(dag_file)

        clusterid, submit_result = condor_queue_job(dag_sub, schedd=schedd)
    except Exception as e:
        print(f'error submitting dag: {e}')
        clusterid = None
        submit_result = None
    return clusterid, submit_result


def condor_rm(spec, schedd):
    """
    Remove jobs from the HTCondor job queue based on the specified criteria.

    This function interacts with an HTCondor schedd (scheduler) to locate and
    remove jobs that match the provided specifications. Jobs can be queried
    either by their cluster ID or by their name, and the matching jobs will
    then be marked for removal from the job queue.

    :param spec: Criteria or specification to identify the jobs that should be
                 removed. This can be its cluster ID or job name used for
                 querying.
    :type spec: Any
    :param schedd: Identifier for the HTCondor scheduler where the jobs are
                   located. Defaults to the local scheduler if not provided.
    :type schedd: str|htcondor.Schedd | None

    :return: None
    """

    theschedd = get_schedd(schedd)

    res_list = query_by_clusterid(spec, schedd=theschedd)
    for res in res_list:
        job_id = res[ 'ClusterId']
        theschedd.act(htcondor.JobAction.Remove, job_id)

    res_list = query_by_name(spec, schedd=theschedd)
    for res in res_list:
        job_id = res[ 'ClusterId']
        theschedd.act(htcondor.JobAction.Remove, job_id)

def condor_get_held_jobs(constraint=None, projection=None, schedd=None):
    """
    Retrieve a list of held jobs from the HTCondor job queue based on the
    specified criteria.

    """
    default_constraint = 'JobStatus == 5'
    defaul_projection = ["Owner", "ClusterId", "ProcId", "JobBatchName", "JobStatus", "HoldReasonCode", "HoldReasonSubCode"
                         "NumJobStarts", "EnteredCurrentStatus", "HoldReason"]

    if constraint is None:
        constraint = default_constraint
    if projection is None:
        projection = defaul_projection

    ret = schedd_query(constraint=constraint, schedd=schedd, projection=projection)
    return ret

def condor_release_held_jobs(constraint=None, max_starts=3, wait=60, user_hold=False, schedd=None):
    """
    Releases held jobs in an HTCondor scheduler based on the reason code and subcode

    This function inspects held jobs in the HTCondor queue according to the input parameters
    and attempts to release them based on predefined criteria. Only jobs satisfying specific
    conditions are considered for release, and timing constraints are enforced to ensure that
    jobs are not prematurely released.

    :param constraint: A custom constraint used to filter held jobs. Defaults to all held jobs.
    :param max_starts: Maximum number of starts for a job before it isn't considered
        releasable. Defaults to 3.
    :param wait: Time (in seconds) to wait if a held job does not meet the minimum
        waiting time since its hold status was entered. Defaults to 60.
    :param user_hold: A boolean that specifies whether jobs held by 'user hold' should
        also be released. Defaults to False.
    :param schedd: Optional. Specifies a particular scheduler to retrieve jobs from.
        If not provided, the default scheduler is used.

    :return: None
    """
    user = getpass.getuser()
    releaseable_hold_reasons = {6:7, 7:2}
    if user_hold:
        releaseable_hold_reasons[1] = 0

    jobs = condor_get_held_jobs(constraint=constraint, schedd=schedd)
    if jobs:
        theschedd = get_schedd(schedd)
        for job in jobs:
            if job['HoldReasonCode'] in releaseable_hold_reasons.keys() and job['NumJobStarts'] < max_starts:
                if job['HoldReasonSubCode'] == releaseable_hold_reasons[job['HoldReasonCode']]:
                    hold_time = time() - job['EnteredCurrentStatus']
                    if hold_time < wait:
                        sleep(wait - hold_time)
                        job_id = job['ClusterId']
                        owner = job['Owner']
                        if owner == user:
                            theschedd.act(htcondor.JobAction.Release, job_id)
                        else:
                            print(f'Job {job_id} is held by {owner}, not {user}. If you have the privileges you could use:')
                            print(f'sudo -u {owner} condor_release {job_id}"')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: nu:ai:ts=4:sw=4

#
#  Copyright (C) 2025 Joseph Areeda <joseph.areeda@ligo.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
""""""

__author__ = 'joseph areeda'
__email__ = 'joseph.areeda@ligo.org'

from omicron_utils.TerminalFormat import format_str, ansi_color_text


def test_format():
    bold = format_str('bold', "bold")
    print(f'\nThis is {bold} text')
    print(f'This is {format_str("reverse", "reverse")} text')


def test_format_color():
    print()
    print(f'Thus is {ansi_color_text("yellow", "yellow")} text')
    print(f'Thus is {ansi_color_text("red", "red")} text')
    print(f'Thus is {ansi_color_text("green", "green")} text')
    print(f'Thus is {ansi_color_text("blue", "blue")} text')
    print(f'Thus is {ansi_color_text("white", "white")} text')
    print(f'Thus is {ansi_color_text("black", "black", "red")} text on red background')
    print(f'Thus is {ansi_color_text("black", "black", "green")} text on green background')
    print(f'Thus is {ansi_color_text("black", "black", "blue")} text on blue background')
    print(f'Thus is {ansi_color_text("black", "black", "yellow")} text on yellow background')
    print(f'Thus is {ansi_color_text("black", "black", "white")} text on white background')
    print(f'Thus is {ansi_color_text("white", "white", "black")} text on black background')
    print(f'This is {ansi_color_text("purple", 0xFF00FF)} text')
    print(f'This is {ansi_color_text("red", 0xFF0000)} text')
    print(f'This is {ansi_color_text("red", 0xFF0000, 0xFFFFFF)} text on white background')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: nu:ai:ts=4:sw=4

#
#  Copyright (C) 2025 Joseph Areeda <joseph.areeda@ligo.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

""""""

import time

from omicron_utils.TerminalFormat import ansi_color_text
from omicron_utils.omicron_config import OmicronConfig

start_time = time.time()

import argparse
import logging
from pathlib import Path
import sys
import traceback

try:
    from ._version import __version__
except ImportError:
    __version__ = '0.0.0'

__author__ = 'joseph areeda'
__email__ = 'joseph.areeda@ligo.org'
__process_name__ = Path(__file__).name

logger = None


def parser_add_args(parser):
    """
    Set up command parser
    :param argparse.ArgumentParser parser:
    :return: None but parser object is updated
    """
    parser.add_argument('-v', '--verbose', action='count', default=1,
                        help='increase verbose output')
    parser.add_argument('-V', '--version', action='version',
                        version=__version__)
    parser.add_argument('-q', '--quiet', default=False, action='store_true',
                        help='show only fatal errors')
    parser.add_argument('--config', type=Path, help='path to config file')
    parser.add_argument('command', type=str, help='command to run',
                        choices=['write', 'merge', 'diff', 'print'])

    epilog =\
        f"""
        By default, most LIGO omicron process get their parameters from a configuration file. We are still working
        to move any internal defaults to the config file. The hierarchy of parameter is 1) The internal defaults,
        2) the external config file by default {OmicronConfig.get_default_config_path()} or specified as acommand
        line argument 3) any relevant command line arguments.

        This program is meant to aid in maintaining the external config file as we update the programs,
        adding options and changing defaults.
        """
    parser.epilog = epilog


def internal_color(string):
    """
    Adds terminal control codes to idenntify string as belonging to internal (default) config
    :param str string: string to colorize
    :return str: colorized string
    We currentl use blue text on black background
    """
    ret = ansi_color_text(string, 'blue', 'black', True)
    return ret


def external_color(string):
    """
    Adds terminal control codes to idenntify string as belonging to external (Editable) config
    :param str string: string to colorize
    :return str: colorized string
    We currentl use green text on black background
    """
    ret = ansi_color_text(string, 'green', 'black', True)
    return ret


def show_diff(internal_config, current_config):
    """
    Display difference between internal and external configurations
    :param OmicronConfig internal_config: our defaults
    :param  OmicronConfig current_config: external config usually the default one
    :return bool: True if objects are different
    """
    print(f'Comparing {internal_color("Internal defaults")} with {external_color(current_config.path)}')
    internal_sections = list(internal_config.config.sections())
    internal_sections.sort()
    external_sections = list(current_config.config.sections())
    external_sections.sort()
    common_sections = list(set(internal_sections).intersection(external_sections))
    internal_only_sections = list(set(internal_sections) - set(common_sections))
    external_only_sections = list(set(external_sections) - set(common_sections))
    if internal_only_sections:
        for section in internal_only_sections:
            print(f'{internal_color(section)} only in internal config')
    if external_only_sections:
        for section in external_only_sections:
            print(f'{external_color(section)} only in external config')

    for section in common_sections:
        internal_options = list(internal_config.config.options(section))
        internal_options.sort()
        external_options = list(current_config.config.options(section))
        external_options.sort()

        internal_only_options = list(set(internal_options) - set(external_options))
        external_only_options = list(set(external_options) - set(internal_options))
        common_options = list(set(internal_options).intersection(external_options))

        if internal_only_options:
            print(f'Section {section} has these options in internal config only:')
            for opt in internal_only_options:
                opt_txt = f'   {opt} = {internal_config.get_option(section, opt)}'
                print(internal_color(opt_txt))

        if external_only_options:
            print(f'Section {section} has these options in external config only:')
            for opt in external_only_options:
                opt_txt = f'   {opt} = {current_config.get_option(section, opt)}'
                print(external_color(opt_txt))

        need_section_title = True
        for opt in common_options:
            internal_value = internal_config.get_option(section, opt)
            current_value = current_config.get_option(section, opt)
            if internal_value != current_value:
                if need_section_title:
                    need_section_title = False
                    print(f'Section {section} has differences in these options:')
                current_text = f'  file:       {opt} = {current_value}'
                internal_text = f' internal:   {opt} = {internal_value}'
                print(f'{internal_color(internal_text)}')
                print(f'{external_color(current_text)}')

        if not need_section_title:
            print('---')


def main():
    global logger

    log_file_format = "%(asctime)s - %(levelname)s - %(funcName)s %(lineno)d: %(message)s"
    log_file_date_format = '%m-%d %H:%M:%S'
    logging.basicConfig(format=log_file_format, datefmt=log_file_date_format)
    logger = logging.getLogger(__process_name__)
    logger.setLevel(logging.DEBUG)

    parser = argparse.ArgumentParser(description=__doc__, prog=__process_name__,
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_add_args(parser)
    args = parser.parse_args()
    verbosity = 0 if args.quiet else args.verbose

    if verbosity < 1:
        logger.setLevel(logging.CRITICAL)
    elif verbosity < 2:
        logger.setLevel(logging.INFO)
    else:
        logger.setLevel(logging.DEBUG)

    # debugging?
    logger.debug(f'{__process_name__} version: {__version__} called with arguments:')
    for k, v in args.__dict__.items():
        logger.debug('    {} = {}'.format(k, v))

    internal_config_path = Path(OmicronConfig.get_internal_config_path())
    if internal_config_path is None or not internal_config_path.exists():
        raise FileNotFoundError(f'The internal (default) configuration file {internal_config_path} does not exist')

    if args.config:
        current_config_path = Path(args.config)
        if not current_config_path.exists():
            raise FileNotFoundError(f'Specified configuration: {current_config_path} does not exist')
    else:
        current_config_path = Path(OmicronConfig.get_default_config_path())
        if not current_config_path.exists():
            logger.critical(f'Current configuration: {current_config_path} does not exist, it will be created with '
                            f'default values')
    if current_config_path.exists():
        current_config = OmicronConfig(current_config_path, logger=logger)
    else:
        current_config = OmicronConfig(logger=logger)
    current_config.get_config(save_if_none=True)

    internal_config = OmicronConfig(internal_config_path, logger=logger)
    internal_config.get_config(save_if_none=False)

    if not current_config_path.exists():
        logger.critical(f'Specified configuration: {current_config_path} still does not exist')
    else:
        current_config = OmicronConfig(current_config_path, logger=logger)
        current_config.get_config(save_if_none=False)

        if args.command.lower() == 'print':
            current_config.print_config()
        elif args.command.lower() == 'merge':
            pass
        elif args.command.lower() == 'diff':
            show_diff(internal_config, current_config)
        else:
            pass


if __name__ == "__main__":
    try:
        main()
    except (ValueError, TypeError, OSError, NameError, ArithmeticError, RuntimeError) as ex:
        print(ex, file=sys.stderr)
        traceback.print_exc(file=sys.stderr)

    if logger is None:
        logging.basicConfig()
        logger = logging.getLogger(__process_name__)
        logger.setLevel(logging.DEBUG)
    # report our run time
    logger.info(f'Elapsed time: {time.time() - start_time:.1f}s')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: nu:ai:ts=4:sw=4

#
#  Copyright (C) 2024 Joseph Areeda <joseph.areeda@ligo.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
""""""

__author__ = 'joseph areeda'
__email__ = 'joseph.areeda@ligo.org'

import logging
from math import sqrt

import numpy as np
from matplotlib import use, pyplot
use('agg')


def histogram_plot(data, outfile, suptitle='', title='', xscale=None, bins=None, logger=None):
    """

    :param data:
    :param outfile:
    :param suptitle:
    :param title:
    :param xscale:
    :param bins:
    :param logger:
    :return:
    """

    if logger is None:
        logging.basicConfig()
        logger = logging.getLogger('histogram_plot')
        logger.setLevel(logging.CRITICAL)

    # plot histogram
    pyplot.figure(figsize=(24, 15))
    xmin = data.min()
    xmax = data.max()
    if xscale:
        if '%' in xscale:
            pct = float(xscale.replace('%', ''))
            plt_xmax = np.percentile(data, pct)
        else:
            plt_xmax = float(xscale)
    else:
        plt_xmax = xmax
    logger.info(f'Plot max: {plt_xmax:.1f}')
    new_data = data[data <= plt_xmax]
    nbins = bins if bins else int(max(10, round(sqrt(len(new_data)))))
    logger.info(f'nbins: {nbins}')
    pyplot.hist(new_data, nbins)

    ax = pyplot.gca()
    if suptitle:
        pyplot.suptitle(suptitle)
    if title:
        ax.set_title(title, fontsize=10, y=.97)
    ax.set_xlabel('Time (s) last gps to file write')
    ax.set_ylabel('N')

    ax.set_xlim(xmin, plt_xmax)
    pyplot.savefig(outfile)
    logger.info(f'Saved histogram as: {str(outfile.absolute())}')

    return outfile



 #!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: nu:ai:ts=4:sw=4

#
#  Copyright (C) 2025 Joseph Areeda <joseph.areeda@ligo.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
""""""

__author__ = 'joseph areeda'
__email__ = 'joseph.areeda@ligo.org'

import shutil
import socket
import tempfile
from pathlib import Path

import pytest
from htcondor2 import HTCondorException

from omicron_utils.condor_utils import get_schedd, condor_submit_job, wait_for_job, query_by_clusterid, \
    condor_submit_dag, condor_rm


class TestCondorUtils:

    @classmethod
    def setup_class(self):
        fqdn = socket.getfqdn()
        print(f'\nRunning tests on {fqdn} cwd: {Path.cwd()}')

        self.test_data_dir = Path('tests/test_data').absolute()
        self.sleep_submit = self.test_data_dir / 'sleep.submit'
        self.dag_submit = self.test_data_dir / 'sleep4.dag'
        self.dag_submit2 = self.test_data_dir / 'sleep2.dag'
        ok = True

        try:
            get_schedd()
        except HTCondorException as e:
            print(f'\nNo schedd found {e}: skipping test')
            ok = False
        for test_file in [self.sleep_submit, self.dag_submit, self.dag_submit2]:
            if not test_file.exists():
                print(f'Test file {test_file.absolute} does not exist')
                ok = False
        assert ok, 'No schedd available or test files missing'
        pass


    def test_condor_submit_job(self):

        my_fqdn = socket.getfqdn()
        print(f'\nMy FQDN: {my_fqdn}')
        run_test = True
        try:
            get_schedd()
        except HTCondorException as e:
            print(f'\nNo schedd found {e}: skipping test')
            run_test = False
        if run_test:
            with tempfile.TemporaryDirectory() as tmpdirname:
                print(f'\nTemporary directory created at: {tmpdirname}')
                sub_src = self.sleep_submit
                subfile = Path(tmpdirname) / sub_src.name
                shutil.copy(sub_src, subfile)
                print(f'Submit file: {subfile}')
                job_id, res = condor_submit_job(str(subfile.absolute()))
                print(f'Job ID: {job_id}')
                q_result = query_by_clusterid(job_id)
                print(f'Job {job_id} in queue: {q_result}')
                assert len(q_result) == 1
                assert job_id == q_result[0]['ClusterId']
                with pytest.raises(TimeoutError):
                    wait_for_job(job_id, timeout=10, sleep_time=2)
                print(f'{job_id}, appropriate timeout received´.')

                self.wait_for_it(job_id)
                print(f'Job {job_id} completed')

    def test_condor_submit_dag(self):

        my_fqdn = socket.getfqdn()
        print(f'\n FQDN: {my_fqdn}')

        run_test = True
        try:
            get_schedd()
        except HTCondorException as e:
            print(f'\nNo schedd found {e}: skipping test')
            run_test = False
        if run_test:
            with tempfile.TemporaryDirectory() as tmpdirname:
                print(f'\nTemporary directory created at: {tmpdirname}')
                sub_src = self.dag_submit
                subfile = Path(tmpdirname) / sub_src.name
                shutil.copy(sub_src, subfile)
                print(f'Submit file: {subfile}')
                job_id, res = condor_submit_dag(str(subfile.absolute()))
                print(f'Job ID: {job_id}')
                assert job_id is not None, "condor_submit_dag did not return a job ID"
                q_result = query_by_clusterid(job_id)
                print(f'Job {job_id} in queue: {q_result}')
                assert q_result is not None and len(q_result) > 0, 'Jo not found in queue'
                assert job_id == q_result[0]['ClusterId']
                with pytest.raises(TimeoutError):
                    wait_for_job(job_id, timeout=10, sleep_time=2)
                self.wait_for_it(job_id)
                print(f'Job {job_id} completed')

    def test_condor_submit_dag2(self):

        my_fqdn = socket.getfqdn()
        print(f'\n FQDN: {my_fqdn}')

        run_test = True
        try:
            get_schedd()
        except HTCondorException as e:
            print(f'\nNo schedd found {e}: skipping test')
            run_test = False
        if run_test:
            with tempfile.TemporaryDirectory() as tmpdirname:
                print(f'\nTemporary directory created at: {tmpdirname}')
                dag_sub_src = self.dag_submit2
                dag_subfile = Path(tmpdirname) / dag_sub_src.name
                shutil.copy(dag_sub_src, dag_subfile)
                print(f'DAG submit file: {dag_subfile}')
                job_sub_src = self.sleep_submit
                job_subfile = Path(tmpdirname) / job_sub_src.name
                shutil.copy(job_sub_src, job_subfile)
                print(f'Job submit file: {job_subfile}')
                job_src = self.sleep_submit
                job_id, res = condor_submit_dag(str(dag_subfile.absolute()))
                print(f'Job ID: {job_id}')
                assert job_id is not None, "condor_submit_dag did not return a job ID"
                q_result = query_by_clusterid(job_id)
                print(f'Job {job_id} in queue: {q_result}')
                assert q_result is not None and len(q_result) > 0, 'Jo not found in queue'
                assert job_id == q_result[0]['ClusterId']
                with pytest.raises(TimeoutError):
                    wait_for_job(job_id, timeout=10, sleep_time=2)
                self.wait_for_it(job_id)
                print(f'Job {job_id} completed')

    def wait_for_it(self, job_id, timeout=250, sleep_time=5):

        try:
            wait_for_job(job_id, timeout=timeout, sleep_time=sleep_time)
        except TimeoutError as te:
            condor_rm(job_id)
            pytest.fail(f'Job {job_id} did not complete in time')

    def test_release_held_jobs(self):
        pass